import numpy as np
import pytest
from scipy.sparse import dok_array

from mnp.preparation.aggregate_land_type_map import LandTypeMap, aggregate_row


@pytest.fixture
def unaggregated_row():
    return np.array(
        [
            [0, 0, 1, 0, 2, 2, 3, 3],
            [0, 0, 0, 0, 0, 0, 3, 1],
        ]
    ).T.flatten()


class TestLandTypeMap:
    @pytest.fixture
    def land_type_map(self, request):
        return LandTypeMap(
            request.path.parent / "data/rasters/land_types.tif",
            request.path.parent / "data/rasters/land_types.csv",
            request.path.parent / "data/rasters/environmentals/",
        )

    def test_aggregate_multiprocess(self, land_type_map):
        land_type_map.aggregate_multiprocess()
        assert land_type_map.land_types[2][549, 0] == 1  # left border (lt 2)
        assert land_type_map.land_types[2][388, 86] == 0  # (lt 1)
        assert land_type_map.land_types[1][563, 161] == 0  # (no lt)
        assert land_type_map.land_types[1][578, 639] == 1  # right border (lt 1)


def test_aggregate_row(unaggregated_row):
    lt_dict = {
        k: dok_array(
            (
                1,
                4,
            ),
            dtype=np.double,
        )
        for k in [1, 2, 3, 4]
    }
    unrecognized = set()
    land_types, unrecognized_land_type_ids = aggregate_row(
        unaggregated_row, 0, lt_dict, unrecognized, 2, nodata_array=np.zeros((2, 2))
    )
    np.testing.assert_array_equal(
        land_types[1].todense(), np.array([[0, 0.25, 0, 0.25]])
    )
    np.testing.assert_array_equal(land_types[2].todense(), np.array([[0, 0, 0.5, 0]]))
    np.testing.assert_array_equal(land_types[3].todense(), np.array([[0, 0, 0, 0.75]]))
