from configparser import ConfigParser
from pathlib import Path

import pandas as pd
import pytest

from mnp.MNP import mnp


@pytest.fixture(scope="module")
def config(request, tmp_path_factory):
    config_dict = ConfigParser()
    config_dict.read(request.path.parent / "data/config.ini")
    config_dict["IO"]["cover_directory"] = str(
        tmp_path_factory.mktemp("mnp_test_run").resolve()
    )
    config_dict["traits"]["species_names"] = str(
        request.path.parent / config_dict["traits"]["species_names"]
    )
    config_dict["traits"]["species_traits"] = str(
        request.path.parent / config_dict["traits"]["species_traits"]
    )
    config_dict["traits"]["group_traits"] = str(
        request.path.parent / config_dict["traits"]["group_traits"]
    )
    config_dict["land_types"]["source"] = str(
        request.path.parent / config_dict["land_types"]["source"]
    )
    config_dict["land_types"]["table"] = str(
        request.path.parent / config_dict["land_types"]["table"]
    )
    config_dict["land_types"]["suitability_indexes"] = str(
        request.path.parent / config_dict["land_types"]["suitability_indexes"]
    )
    config_dict["environmental_1"]["raster"] = str(
        request.path.parent / config_dict["environmental_1"]["raster"]
    )
    config_dict["environmental_1"]["table"] = str(
        request.path.parent / config_dict["environmental_1"]["table"]
    )
    config_dict["environmental_2"]["raster"] = str(
        request.path.parent / config_dict["environmental_2"]["raster"]
    )
    config_dict["environmental_2"]["table"] = str(
        request.path.parent / config_dict["environmental_2"]["table"]
    )
    config_dict["species_subselection_1"]["table"] = str(
        request.path.parent / config_dict["species_subselection_1"]["table"]
    )
    config_dict["species_subselection_2"]["table"] = str(
        request.path.parent / config_dict["species_subselection_2"]["table"]
    )

    return config_dict


@pytest.fixture(scope="module")
def output_dir(config):
    return Path(config["IO"]["cover_directory"]) / "output"


@pytest.fixture(scope="module", autouse=True)
def run_mnp(config):
    mnp(config)


def test_detailed_table(request, output_dir):
    expected_table = pd.read_csv(
        request.path.parent / "data/expected/detailed_table.csv"
    )
    actual_table = pd.read_csv(
        output_dir / "species_subselections/all/tables/detailed_table.csv"
    )
    pd.testing.assert_frame_equal(actual_table, expected_table, check_dtype=False)


def test_summary_table(request, output_dir):
    expected_table = pd.read_csv(
        request.path.parent / "data/expected/summary_table.csv"
    )
    actual_table = pd.read_csv(
        output_dir / "species_subselections/all/tables/summary_table.csv"
    )
    pd.testing.assert_frame_equal(actual_table, expected_table, check_dtype=False)
